X-Original-To: pgsql-patches-postgresql.org@localhost.postgresql.org Received: from localhost (unknown [200.46.204.144]) by svr1.postgresql.org (Postfix) with ESMTP id BD8AA52997 for ; Tue, 19 Jul 2005 21:39:50 -0300 (ADT) Received: from svr1.postgresql.org ([200.46.204.71]) by localhost (av.hub.org [200.46.204.144]) (amavisd-new, port 10024) with ESMTP id 16869-05 for ; Wed, 20 Jul 2005 00:39:38 +0000 (GMT) Received: from mail.Mi8.com (mail.mi8.com [63.240.6.41]) by svr1.postgresql.org (Postfix) with ESMTP id E9FF4529A1 for ; Tue, 19 Jul 2005 21:39:36 -0300 (ADT) Received: from 172.16.1.110 by mail.Mi8.com with ESMTP (- GW01 Welcome to Mi8 Corporation www.Mi8.com); Tue, 19 Jul 2005 20:39:33 -0400 X-Server-Uuid: F1A2E19A-84E4-48DD-8F48-B475613F58B2 Received: from MI8NYCMAIL06.Mi8.com ([172.16.1.175]) by D01SMTP01.Mi8.com with Microsoft SMTPSVC(6.0.3790.211); Tue, 19 Jul 2005 20:39:33 -0400 Received: from 67.103.45.218 ([67.103.45.218]) by MI8NYCMAIL06.Mi8.com ( [172.16.1.219]) via Exchange Front-End Server mi8owa.mi8.com ( [172.16.1.106]) with Microsoft Exchange Server HTTP-DAV ; Tue, 19 Jul 2005 20:39:33 -0500 User-Agent: Microsoft-Entourage/11.1.0.040913 Date: Tue, 19 Jul 2005 17:39:33 -0700 Subject: Re: COPY FROM performance improvements From: "Luke Lonergan" To: "Mark Wong" , "Andrew Dunstan" Cc: "Alon Goldshuv" , pgsql-patches@postgresql.org Message-ID: In-Reply-To: <200507192251.j6JMpAjA016147@smtp.osdl.org> MIME-Version: 1.0 X-OriginalArrivalTime: 20 Jul 2005 00:39:33.0522 (UTC) FILETIME=[7F7B3720:01C58CC3] X-WSS-ID: 6EC342CF1J47631110-01-01 Content-Type: text/plain; charset=us-ascii Content-Transfer-Encoding: 7bit X-Virus-Scanned: by amavisd-new at hub.org X-Spam-Status: No, hits=1.574 tagged_above=0 required=5 tests=AWL, FORGED_RCVD_HELO, RCVD_NUMERIC_HELO X-Spam-Level: * X-Archive-Number: 200507/395 X-Sequence-Number: 16735 Good points on all, another element in the performance expectations is the ratio of CPU speed to I/O subsystem speed, as Alon had hinted earlier. This patch substantially (500%) improves the efficiency of parsing in the COPY path, which, on a 3GHz P4 desktop with a commodity disk drive represents 8 of a total of 30 seconds of processing time. So, by reducing the parsing time from 8 seconds to 1.5 seconds, the overall COPY time is reduced from 30 seconds to 23.5 seconds, or a speedup of about 20%. On a dual 2.2GHz Opteron machine with a 6-disk SCSI RAID subsystem capable of 240MB/s sequential read and writes, the ratios change and we see between 35% and 95% increase in COPY performance, with the bottleneck being CPU. The disk is only running at about 90MB/s during this period. I'd expect that as your CPUs slow down relative to your I/O speed, and Itaniums or IT2s are quite slow, you should see an increased effect of the parsing improvements. One good way to validate the effect is to watch the I/O bandwidth using vmstat 1 (on Linux) while the load is progressing. When you watch that with the unpatched source and with the patched source, if they are the same, you should see no benefit from the patch (you are I/O limited). If you check your underlying sequential write speed, you will be bottlenecked at roughly half that in performing COPY because of the write-through the WAL. - Luke On 7/19/05 3:51 PM, "Mark Wong" wrote: > Whoopsies, yeah good point about the PRIMARY KEY. I'll fix that. > > Mark > > On Tue, 19 Jul 2005 18:17:52 -0400 > Andrew Dunstan wrote: > >> Mark, >> >> You should definitely not be doing this sort of thing, I believe: >> >> CREATE TABLE orders ( >> o_orderkey INTEGER, >> o_custkey INTEGER, >> o_orderstatus CHAR(1), >> o_totalprice REAL, >> o_orderDATE DATE, >> o_orderpriority CHAR(15), >> o_clerk CHAR(15), >> o_shippriority INTEGER, >> o_comment VARCHAR(79), >> PRIMARY KEY (o_orderkey)) >> >> Create the table with no constraints, load the data, then set up primary keys >> and whatever other constraints you want using ALTER TABLE. Last time I did a >> load like this (albeit 2 orders of magnitude smaller) I saw a 50% speedup >> from deferring constarint creation. >> >> >> cheers >> >> andrew >> >> >> >> Mark Wong wrote: >> >>> Hi Alon, >>> >>> Yeah, that helps. I just need to break up my scripts a little to just >>> load the data and not build indexes. >>> >>> Is the following information good enough to give a guess about the data >>> I'm loading, if you don't mind? ;) Here's a link to my script to create >>> tables: >>> http://developer.osdl.org/markw/mt/getfile.py?id=eaf16b7831588729780645b2bb4 >>> 4f7f23437e432&path=scripts/pgsql/create_tables.sh.in >>> >>> File sizes: >>> -rw-r--r-- 1 markw 50 2.3G Jul 8 15:03 customer.tbl >>> -rw-r--r-- 1 markw 50 74G Jul 8 15:03 lineitem.tbl >>> -rw-r--r-- 1 markw 50 2.1K Jul 8 15:03 nation.tbl >>> -rw-r--r-- 1 markw 50 17G Jul 8 15:03 orders.tbl >>> -rw-r--r-- 1 markw 50 2.3G Jul 8 15:03 part.tbl >>> -rw-r--r-- 1 markw 50 12G Jul 8 15:03 partsupp.tbl >>> -rw-r--r-- 1 markw 50 391 Jul 8 15:03 region.tbl >>> -rw-r--r-- 1 markw 50 136M Jul 8 15:03 supplier.tbl >>> >>> Number of rows: >>> # wc -l *.tbl >>> 15000000 customer.tbl >>> 600037902 lineitem.tbl >>> 25 nation.tbl >>> 150000000 orders.tbl >>> 20000000 part.tbl >>> 80000000 partsupp.tbl >>> 5 region.tbl >>> 1000000 supplier.tbl >>> >>> Thanks, >>> Mark >>> >>> On Tue, 19 Jul 2005 14:05:56 -0700 >>> "Alon Goldshuv" wrote: >>> >>> >>> >>>> Hi Mark, >>>> >>>> I improved the data *parsing* capabilities of COPY, and didn't touch the >>>> data conversion or data insertion parts of the code. The parsing >>>> improvement >>>> will vary largely depending on the ratio of parsing -to- converting and >>>> inserting. >>>> >>>> Therefore, the speed increase really depends on the nature of your data: >>>> >>>> 100GB file with >>>> long data rows (lots of parsing) >>>> Small number of columns (small number of attr conversions per row) >>>> less rows (less tuple insertions) >>>> >>>> Will show the best performance improvements. >>>> >>>> However, same file size 100GB with >>>> Short data rows (minimal parsing) >>>> large number of columns (large number of attr conversions per row) >>>> AND/OR >>>> more rows (more tuple insertions) >>>> >>>> Will show improvements but not as significant. >>>> In general I'll estimate 40%-95% improvement in load speed for the 1st case >>>> and 10%-40% for the 2nd. But that also depends on the hardware, disk speed >>>> etc... This is for TEXT format. As for CSV, it may be faster but not as >>>> much >>>> as I specified here. BINARY will stay the same as before. >>>> >>>> HTH >>>> Alon. >>>> >>>> >>>> >>>> >>>> >>>> >>>> On 7/19/05 12:54 PM, "Mark Wong" wrote: >>>> >>>> >>>> >>>>> On Thu, 14 Jul 2005 17:22:18 -0700 >>>>> "Alon Goldshuv" wrote: >>>>> >>>>> >>>>> >>>>>> I revisited my patch and removed the code duplications that were there, >>>>>> and >>>>>> added support for CSV with buffered input, so CSV now runs faster too >>>>>> (although it is not as optimized as the TEXT format parsing). So now >>>>>> TEXT,CSV and BINARY are all parsed in CopyFrom(), like in the original >>>>>> file. >>>>>> >>>>>> >>>>> Hi Alon, >>>>> >>>>> I'm curious, what kind of system are you testing this on? I'm trying to >>>>> load 100GB of data in our dbt3 workload on a 4-way itanium2. I'm >>>>> interested in the results you would expect. >>>>> >>>>> Mark >>>>> >>>>> >>>>> >>> > > ---------------------------(end of broadcast)--------------------------- > TIP 6: explain analyze is your friend >